unit uaesthread; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, uaes;
  
  
type TAESThread = class(TThread)
protected
  FKey,FSourcePath,FDestPath:String;
  FKeySize:TAESKeySize;
  syncPInfo: Integer;
  syncM: String;
  syncOverWrite,
  Canceled,
  FEncode: Boolean;
  procedure SPI(Progress:Integer); stdcall; //callback Prozedur Fortschritt
  procedure syncSPI;
  procedure CheckCancel(var Cancel:Boolean); stdcall; //callback Prozedur Abbruch
  procedure MOut(M:String); stdcall; //callback Prozedur Meldung
  procedure syncMOut;
  procedure Execute; override;
public
  destructor Destroy; override;
  procedure Cancel;
  property Key:String read FKey write FKey;
  property KeySize:TAESKeySize read FKeySize write FKeySize default ks128;
  property SourcePath:String read FSourcePath write FSourcePath;
  property DestPath:String read FDestPath write FDestPath;
  property Encode:Boolean read FEncode write FEncode default TRUE;
end;

implementation

uses uMain;

procedure TAESThread.Execute;
var AES : TAES;
begin
  Canceled := FALSE;
  AES := TAES.Create(FKey,FKeySize); // TAES-Instanz wird erzeugt
  try
    // Datei ver-/entschlsseln
    AES.EnDecodeFile(FSourcePath,FDestPath,FEncode,@SPI,@CheckCancel,@MOut);
  finally
    AES.Free; // Ressourcen freigeben
  end;
end;

destructor TAESThread.Destroy;
begin
  synchronize(@Form1.WorkDone);
  inherited;
end;

procedure TAESThread.SPI(Progress:Integer); stdcall;
begin
  syncPInfo := Progress;
  synchronize(@syncSPI);
end;

procedure TAESThread.syncSPI;
begin
  Form1.SetPI(syncPInfo);
end;

procedure TAESThread.CheckCancel(var Cancel:Boolean); stdcall;
begin
  Cancel := Canceled;
end;

procedure TAESThread.Cancel;
begin
  Canceled := TRUE;
end;

procedure TAESThread.MOut(M:String); stdcall;
begin
  if M<>'' then syncM := '- '+M
  else syncM := '';
  synchronize(@syncMOut);
end;

procedure TAESThread.syncMOut;
begin
  Form1.LogAppend(syncM);
end;


end.

