unit uMain; 

{$mode objfpc}{$H+}

interface

uses
  {$IFDEF WIN32} Windows, {$ENDIF}
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, Buttons, ComCtrls, uaes, uaesthread;

type

  { TForm1 }

  TForm1 = class(TForm)
    bQuelle: TButton;
    bStart: TButton;
    bZiel: TButton;
    cbIdle: TCheckBox;
    editHash: TEdit;
    editKey: TEdit;
    editQuelle: TEdit;
    editZiel: TEdit;
    lKey: TLabel;
    lPass: TLabel;
    Log: TMemo;
    OpenDialog: TOpenDialog;
    PBar: TProgressBar;
    rgKeySize: TRadioGroup;
    rgMode: TRadioGroup;
    SaveDialog: TSaveDialog;
    procedure Form1Create(Sender: TObject);
    procedure Form1CloseQuery(Sender: TObject; var CanClose: boolean); // Abfrage vor dem Beenden
    procedure bQuelleClick(Sender: TObject); // Quelldatei whlen
    procedure bZielClick(Sender: TObject);   // Zieldatei whlen
    procedure bStartClick(Sender: TObject);  // Starten und Abbrechen
    procedure cbIdleChange(Sender: TObject); // Prozessprioritt ndern
  private
    Working,CloseOnWorkDone:Boolean;
    Zeit:TDateTime;
    procedure Init; // Initialisierung
    procedure Abbrechen; // Vorgang abbrechen
    procedure EnDecode;  // Ver- und Entschlsseln
    { private declarations }
  public
    AEST:TAESThread; // Verschlsselungs-Thread
    procedure SetPI(ProgressInfo:Integer); // Fortschrittsanzeige steuern
    procedure LogAppend(M:String);         // Statusmeldung anzeigen
    procedure WorkDone;                    // Manahmen nach der Verschlsselung
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

uses usha256,umisc;

const AES_Endung = '.aes';

{ TForm1 }

procedure TForm1.Form1Create(Sender: TObject);
begin
  {$IFNDEF WIN32}
  cbIdle.Visible := FALSE;
  cbIdle.Enabled := FALSE;
  {$ENDIF}
  CloseOnWorkDone := FALSE;
  Init;
  LogAppend('Advanced Encryption Standard (AES) - Implementierung mit FreePascal');
  LogAppend('Autor: Henner Heck');
end;

procedure TForm1.Form1CloseQuery(Sender: TObject; var CanClose: boolean);
begin
  if Working then
    begin
      if (MessageDlg('Mchten Sie abbrechen und das Programm beenden?'
                     ,mtConfirmation,[mbYes,mbNo],0) <> mrYes) then
        begin
          CanClose := FALSE;
        end
      else
        begin
          if Working then
            begin
              CanClose := FALSE;
              CloseOnWorkDone := TRUE;
              Abbrechen;
            end;
        end;
    end;
end;

procedure TForm1.bQuelleClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    begin
      editQuelle.Text := OpenDialog.FileName;
      if Lowercase(ExtractFileExt(editQuelle.Text)) = AES_Endung then
        begin
          editZiel.Text := ChangeFileExt(editQuelle.Text,'');
          rgMode.ItemIndex := 1;
        end
      else
        begin
          editZiel.Text   := OpenDialog.FileName +AES_Endung;
          rgMode.ItemIndex := 0;
        end;
    end;
end;

procedure TForm1.bZielClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    editZiel.Text := SaveDialog.FileName;
end;

procedure TForm1.bStartClick(Sender: TObject);
begin
  if (NOT Working) then
    begin
      EnDecode;
      bStart.Caption := 'Verschlsselung/Entschlsselung abbrechen';
    end
  else
    begin
      Abbrechen;
    end;
end;

procedure TForm1.cbIdleChange(Sender: TObject);
begin
  {$IFDEF WIN32}
  if cbIdle.Checked then
    SetPriorityClass(GetCurrentProcess,IDLE_PRIORITY_CLASS)
  else
    SetPriorityClass(GetCurrentProcess,NORMAL_PRIORITY_CLASS);
  {$ENDIF}
end;

procedure TForm1.SetPI(ProgressInfo:Integer);
begin
  PBar.Position := ProgressInfo;
end;

procedure TForm1.LogAppend(M:String);
begin
  Log.Append(M);
end;

procedure TForm1.EnDecode;
var Schluessel:String;
    Weitermachen: Boolean;
begin
  Weitermachen := TRUE;
  editQuelle.Text := Trim(editQuelle.Text);
  editZiel.Text   := Trim(editZiel.Text);
  if FileExists(editZiel.Text) then
    Weitermachen :=
      (MessageDlg('Die Zieldatei existiert bereits, wollen Sie die Datei berschreiben?'
                                                  ,mtConfirmation,[mbYes,mbNo],0) = mrYes);
  if NOT Weitermachen then
    begin
      Init;
    end
  else
    begin
      Working := TRUE;
      Zeit := Now;
      Schluessel := HashStringSHA256(editKey.Text,FALSE); // Passwort wird gehasht
      AEST := TAESThread.Create(TRUE); // Thread wird erzeugt
      with AEST do
        begin
          FreeOnTerminate := TRUE; // Thread ruft Destroy auf, wenn beendet
          case rgKeySize.ItemIndex of // Schlssellnge wird angepasst
            1:   begin KeySize := ks192; SetLength(Schluessel,192 div 8); end;
            2:   begin KeySize := ks256; end;
            else begin KeySize := ks128; SetLength(Schluessel,128 div 8); end;
          end;
          // Schlssel fr die Testdaten
          //KeySize := ks128; Schluessel := #$00#$01#$02#$03#$04#$05#$06#$07#$08#$09#$0a#$0b#$0c#$0d#$0e#$0f;
          //KeySize := ks192; Schluessel := #$00#$01#$02#$03#$04#$05#$06#$07#$08#$09#$0a#$0b#$0c#$0d#$0e#$0f#$10#$11#$12#$13#$14#$15#$16#$17;
          //KeySize := ks256; Schluessel := #$00#$01#$02#$03#$04#$05#$06#$07#$08#$09#$0a#$0b#$0c#$0d#$0e#$0f#$10#$11#$12#$13#$14#$15#$16#$17#$18#$19#$1a#$1b#$1c#$1d#$1e#$1f;
          Key := Schluessel; // Schlssel wird gesetzt
          SourcePath := editQuelle.Text;
          DestPath   := editZiel.Text;
          Encode     := (rgMode.ItemIndex=0); // verschlsseln oder entschlsseln
        end;
      editHash.Text := StringToHex(Schluessel); // Schlssel als HEX anzeigen
      AEST.Resume; // Thread wird gestartet
    end;
end;

procedure TForm1.Init;
begin
  SetPI(0);
  AEST := NIL;
  bStart.Caption := 'Starte Verschlsselung/Entschlsselung';
end;

procedure TForm1.Abbrechen;
begin
  if AEST<>NIL then
    begin
      AEST.Cancel;
    end;
end;

procedure TForm1.WorkDone;
var sZeit:String;
begin
  Zeit := Now -Zeit;
  DateTimeToString(sZeit,'hh.mm.ss.zzz',Zeit);
  LogAppend('- Dauer: ' +sZeit +' h');
  Init;
  Working := FALSE;
  if CloseOnWorkDone then Close;
end;

initialization
  {$I umain.lrs}

end.

